using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Diagnostics;

namespace Rasterbater
{
    public partial class Form1 : Form
    {

        protected PotraceController controller = new PotraceController();


        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            //Load output format providers
            string[] providers = Enum.GetNames(typeof(BackendProviders));
            foreach (string provider in providers)
            {
                vectorizationProvider.Items.Add(provider);
            }
            vectorizationProvider.SelectedIndex = 0;

            string[] policies = Enum.GetNames(typeof(TurnPolicies));
            foreach (string policy in policies)
            {
                turnPolicy.Items.Add(policy);
            }
            turnPolicy.SelectedIndex = 0;

        }


        #region Input File Group Related


        private void inputFilesListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            inputFilesListBox.SelectedIndex = -1; //Do not allow selection.
        }

        private void clearFilesButton_Click(object sender, EventArgs e)
        {
            inputFilesListBox.Items.Clear();
        }

        private void addFilesButton_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                foreach (string fileName in openFileDialog1.FileNames)
                {
                    inputFilesListBox.Items.Add(fileName);
                }
            }
        }

        private void pickOutputDirectory_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                outputDirectory.Text = folderBrowserDialog1.SelectedPath;
            }
        }

        #endregion Input File Group Related

        #region Algorithm Options Group

        private void vectorizationProvider_SelectedIndexChanged(object sender, EventArgs e)
        {
            controller.VectorizationProvider = (BackendProviders)Enum.Parse(typeof(BackendProviders), vectorizationProvider.Text.ToString());
        }

        private void turnPolicy_SelectedIndexChanged(object sender, EventArgs e)
        {
            controller.TurnPolicy = (TurnPolicies)Enum.Parse(typeof(TurnPolicies), turnPolicy.Text.ToString());
        }

        private void turdSize_TextChanged(object sender, EventArgs e)
        {
            int.TryParse(turdSize.Text, out controller.TurdSize);
        }

        private void textbox_RestrictInteger(object sender, KeyPressEventArgs e)
        {
            int bleh = 0;
            if (!int.TryParse(e.KeyChar.ToString(), out bleh)) e.Handled = true;

            if (e.KeyChar == (char)Keys.Back) e.Handled = false;
            if (e.KeyChar == (char)Keys.End) e.Handled = false;
            if (e.KeyChar == (char)Keys.Home) e.Handled = false;
        }

        private void textbox_RestrictDecimal(object sender, KeyPressEventArgs e)
        {
            int bleh = 0;
            if (!int.TryParse(e.KeyChar.ToString(), out bleh)) e.Handled = true;

            if (e.KeyChar == '.') e.Handled = false;

            if (e.KeyChar == (char)Keys.Back) e.Handled = false;
            if (e.KeyChar == (char)Keys.End) e.Handled = false;
            if (e.KeyChar == (char)Keys.Home) e.Handled = false;
        }

        private void alphaMax_TextChanged(object sender, EventArgs e)
        {
            double.TryParse(alphaMax.Text, out controller.AlphaMax);
        }

        private void optTolerance_TextChanged(object sender, EventArgs e)
        {
            double.TryParse(optTolerance.Text, out controller.CurveOptimizationTolerance);
        }

        private void unitsPerPixel_TextChanged(object sender, EventArgs e)
        {
            int.TryParse(unitsPerPixel.Text, out controller.UnitsPerPixel);
        }

        private void longCurve_CheckedChanged(object sender, EventArgs e)
        {
            controller.DisableCurveOptimization = longCurve.Checked;
        }

        #endregion Algorithm Options Group

        #region Output Options

        private void lineColor_TextChanged(object sender, EventArgs e)
        {
            lineColor.Text = lineColor.Text.ToUpper();
            controller.Color = lineColor.Text;
        }

        private void fillColor_TextChanged(object sender, EventArgs e)
        {
            fillColor.Text = fillColor.Text.ToUpper();
            controller.FillColor = fillColor.Text;
        }

        private void opaque_CheckedChanged(object sender, EventArgs e)
        {
            controller.ConvertWhiteShapresToOpaque = opaque.Checked;
        }

        private void group_CheckedChanged(object sender, EventArgs e)
        {
            controller.GroupRelatedPathsTogether = group.Checked;
        }

        private void textbox_RestrictHex(object sender, KeyPressEventArgs e)
        {
            int bleh = 0;
            if (!int.TryParse(e.KeyChar.ToString(), out bleh)) e.Handled = true;

            if (e.KeyChar == '#') e.Handled = false; //# sign.

            string alphas = "ABCDEFabcdef";
            if (alphas.Contains(e.KeyChar.ToString())) e.Handled = false;


            if (e.KeyChar == (char)Keys.Back) e.Handled = false;
            if (e.KeyChar == (char)Keys.End) e.Handled = false;
            if (e.KeyChar == (char)Keys.Home) e.Handled = false;
        }

        #endregion Output Options


        #region Front end options

        private void invert_CheckedChanged(object sender, EventArgs e)
        {
            controller.InvertBitmap = invert.Checked;
        }

        private void blackLevel_TextChanged(object sender, EventArgs e)
        {
            double.TryParse(blackLevel.Text, out controller.BlackLevel);
        }

        #endregion Front end options

        #region Postscript / Eps options

        private void pageSize_TextChanged(object sender, EventArgs e)
        {
            controller.PageSize = pageSize.Text;
        }

        private void clearText_CheckedChanged(object sender, EventArgs e)
        {
            controller.DoNotCompressOutput = clearText.Checked;
        }

        private void levelTwoCompress_CheckedChanged(object sender, EventArgs e)
        {
            controller.UsePostscriptLevelTwoCompression = levelTwoCompress.Checked;
        }

        private void levelThreeCompress_CheckedChanged(object sender, EventArgs e)
        {
            controller.UsePostscriptLevelThreeCompression = levelThreeCompress.Checked;
        }

        private void longCoding_CheckedChanged(object sender, EventArgs e)
        {
            controller.DoNotOptimizeForFileSize = longCoding.Checked;
        }

        #endregion Postscript / Eps options

        #region PGM Options

        private void gammaValue_TextChanged(object sender, EventArgs e)
        {
            double.TryParse(gammaValue.Text, out controller.GammaLevel);
        }

        #endregion PGM Options


        #region Load / Save settings

        private void saveSettings_Click(object sender, EventArgs e)
        {
            if (saveSettingsDialog.ShowDialog() == DialogResult.OK)
            {
                SaveApplicationSettings(saveSettingsDialog.FileName);
            }
        }

        private void SaveApplicationSettings(string fileName)
        {
            FileStream fs = new FileStream(fileName, FileMode.Create);
            BinaryWriter bw = new BinaryWriter(fs);

            try
            {
                Stack controlStack = new Stack();
                controlStack.Push(this);

                while (controlStack.Count > 0)
                {
                    //Get controls
                    Control parentControl = controlStack.Pop() as Control;

                    if (parentControl != null)
                    {
                        foreach (Control ctrl in parentControl.Controls)
                        {
                            if (ctrl.Controls != null && ctrl.Controls.Count > 0)
                            {
                                //Add this controls subcontrols to be processed.
                                controlStack.Push(ctrl);
                            }

                            if (ctrl.GetType() == typeof(TextBox))
                            {
                                bw.Write(((TextBox)ctrl).Text);
                            }
                            if (ctrl.GetType() == typeof(CheckBox))
                            {
                                bw.Write(((CheckBox)ctrl).Checked);
                            }
                            if (ctrl.GetType() == typeof(ListBox))
                            {
                                bw.Write(((ListBox)ctrl).SelectedIndex);
                            }
                            if (ctrl.GetType() == typeof(ComboBox))
                            {
                                bw.Write(((ComboBox)ctrl).SelectedIndex);
                            }

                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Explode?\n" + ex.Message);
            }
            finally
            {

                bw.Close();
                fs.Close();
            }
        }

        private void loadSettings_Click(object sender, EventArgs e)
        {
            if (openSettingsDialog.ShowDialog() == DialogResult.OK)
            {
                LoadApplicationSettings(openSettingsDialog.FileName);
            }
        }

        private void LoadApplicationSettings(string fileName)
        {
            FileStream fs = new FileStream(fileName, FileMode.Open);
            BinaryReader br = new BinaryReader(fs);

            try
            {
                Stack controlStack = new Stack();
                controlStack.Push(this);

                while (controlStack.Count > 0)
                {
                    //Get controls
                    Control parentControl = controlStack.Pop() as Control;

                    if (parentControl != null)
                    {
                        foreach (Control ctrl in parentControl.Controls)
                        {
                            if (ctrl.Controls != null && ctrl.Controls.Count > 0)
                            {
                                //Add this controls subcontrols to be processed.
                                controlStack.Push(ctrl);
                            }

                            if (ctrl.GetType() == typeof(TextBox))
                            {
                                ((TextBox)ctrl).Text = br.ReadString();
                            }
                            if (ctrl.GetType() == typeof(CheckBox))
                            {
                                ((CheckBox)ctrl).Checked = br.ReadBoolean();
                            }
                            if (ctrl.GetType() == typeof(ListBox))
                            {
                                ((ListBox)ctrl).SelectedIndex = br.ReadInt32();
                            }
                            if (ctrl.GetType() == typeof(ComboBox))
                            {
                                ((ComboBox)ctrl).SelectedIndex = br.ReadInt32();
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Thats probably an invalid settings file or it's old.\n" + ex.Message);
            }
            finally
            {
                br.Close();
                fs.Close();
            }
        }

        #endregion Load / Save settings



        private void convertImagesButton_Click(object sender, EventArgs e)
        {
            if (inputFilesListBox.Items.Count == 0)
            {
                MessageBox.Show("Aren't you forgetting something... like... ohh... input files?");
            }
            else
            {
                //Convert input files to string array
                string[] inputFiles = new string[inputFilesListBox.Items.Count];

                int index = 0;
                foreach (object item in inputFilesListBox.Items)
                {
                    string value = item.ToString();

                    inputFiles[index] = value;

                    index++;
                }

                //Pass the files off as a batch job call
                string result = controller.ProcessBatch(inputFiles, outputDirectory.Text);

                ResultsDisplay xResults = new ResultsDisplay();
                xResults.Message = result;

                xResults.ShowDialog();
            }
        }

        /// <summary>
        /// Open an explorer window to the output directory.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        private void browseButton_Click(object sender, EventArgs e)
        {
            ProcessStartInfo psi = new ProcessStartInfo();

            psi.FileName = "explorer.exe";
            psi.Arguments = outputDirectory.Text;

            Process p = Process.Start(psi);

        }









    }
}